@@ -1,5 +1,5 @@
 """
-Library for extracting assets from Spelunky 2.
+Library for extracting and re-packing assets from Spelunky 2.
 
 The vast majority of the work behind extracting these assets comes
 from the work of SciresM.
@@ -13,5 +13,13 @@ https://gist.github.com/Cloppershy/a32b9139f3d222b5ff5b8b23ffac1aac
 
 Both of these contributions were crucial to this library.
 
+Cloppershy was also instrumental in providing the implementation for
+repacking, as seen here:
+
+https://gist.github.com/Cloppershy/046846e593362a2a7284c28f39899eae
+
+
+Additionally, `iojonmbnmb#8149 ` provided the updated hashing code when
+it was broken in 1.13.0
 """
 
